from dateutil.parser import parse


def parse_dates_in_bid(bid):
    if bid:
        bid['pickupDate'] = parse(bid['pickupDate']).strftime('%Y-%m-%d')
        bid['deliveryDate'] = parse(bid['deliveryDate']).strftime('%Y-%m-%d')
        if bid.get('inviteEmailedAt'):
            bid['inviteEmailedAt'] = parse(bid['inviteEmailedAt']).isoformat()
        for bid_history in bid.get('bidHistories', []):
            bid_history['pickupDate'] = parse(bid_history['pickupDate']).strftime('%Y-%m-%d')
            bid_history['deliveryDate'] = parse(bid_history['deliveryDate']).strftime('%Y-%m-%d')



import os

import requests

from cc_py_commons.loads.map_load_response import execute as map_load_response
from cc_py_commons.utils import json_logger

FREIGHT_HUB_URL = os.environ.get('FREIGHT_HUB_URL')
FREIGHT_HUB_TOKEN = os.environ.get("FREIGHT_HUB_TOKEN")


def execute(load_id, account_id=None):
    if not load_id:
        json_logger.error(account_id, 'Invalid load_id provided', load_id=load_id)
        return None
    
    url = f"{FREIGHT_HUB_URL}/freight/{load_id}"
    token = f"Bearer {FREIGHT_HUB_TOKEN}"
    headers = {
        "Authorization": token
    }
    load = None
    try:
        response = requests.get(url, headers=headers, timeout=30)
        if response.status_code == 200:
            json = response.json()
            load = map_load_response(json)
        else:
            json_logger.warning(account_id, 'Failed to get load from FreightHub',
                                response_status_code=response.status_code, response_text=response.text)

    except Exception as e:
        json_logger.error(account_id, 'Error calling FreightHub API', error=str(e))

    return load



import json

from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils.logger_v2 import logger
from .constants import BOOKING_AGENT_TOPIC_ARN


def send_sns_notification(message):
    if message:
        logger.debug(f"booking_agent.send_sns_notification: Sending message {message} to {BOOKING_AGENT_TOPIC_ARN}")
        try:
            sns_service = SnsService()
            sns_service.send(BOOKING_AGENT_TOPIC_ARN, message.get('subject'), json.dumps(message))
        except Exception as e:
            logger.error(f"booking_agent.send_sns_notification: Failed to send notification message: {e}")
    else:
        logger.info("booking_agent.send_sns_notification: Notification not sent because message is empty or null")



import json
import requests

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.logger_v2 import logger

def execute(filters, page=0, size=20, max_size=False):
	"""
	Fetch a list of matching carriers from carrier-hub.
	Always returns a list.
	"""
	carriers = []
	uri = app_config.CARRIER_HUB_URL + '/carriers'
	query_params = {
		'page': page,
		'size': size,
		'maxSize': max_size
	}
	request_headers = {
		'Authorization': 'Bearer ' + app_config.CARRIER_HUB_AUTH_TOKEN,
		'Content-Type': 'application/json'
	}

	try:
		payload = {'filter': filters}
		response = requests.post(uri, json.dumps(payload), params=query_params, headers=request_headers)

		if response.status_code == 200:
			carriers = response.json()['content']
			logger.debug(f"For filters: {filters} number of carriers found: {len(carriers)}")
		else:
			logger.warning(f"Failed to lookup carriers: {response.text}")
	except Exception as e:
		logger.error(f'Error while listing carriers {e}')
		carriers = []

	return carriers



import requests

from cc_py_commons.utils.logger_v2 import logger
from .constants import BOOKING_AGENT_TOKEN, BOOKING_AGENT_URL


def execute(amazon_order_id):
  url = f"{BOOKING_AGENT_URL}/bids/list"
  token = f"Bearer {BOOKING_AGENT_TOKEN}"
  headers = {
    "Authorization": token
  }
  payload = {
    'amazonOrderIds': [amazon_order_id]
  }

  logger.debug(f"Requesting bid for amazon order {amazon_order_id} from booking-agent: {url}, {headers}")
  response = requests.post(url, headers=headers, json=payload)
  bid = None

  logger.debug(f"bid response: {response}")
  if response.status_code == 200:
    bid = response.json()

  return bid



from unittest import TestCase
from unittest.mock import patch

from cc_py_commons.config.web_socket_action import WebSocketAction
from cc_py_commons.services.web_socket import send_notification
from cc_py_commons.sns.sns_service import SnsService


@patch.object(SnsService, 'send')
class TestSendNotification(TestCase):

	def test_when_required_parameters_are_missing(self, sns_service_send):
		result = send_notification.execute(WebSocketAction.BROADCAST_TO_ACCOUNT,
										   None, None, {'message': 'test'})

		self.assertIsNone(result)
		sns_service_send.assert_not_called()

	def test_invalid_action(self, sns_service_send):
		result = send_notification.execute('INVALID_ACTION',
										   None, 1234, {'message': 'test'})

		self.assertIsNone(result)
		sns_service_send.assert_not_called()

	def test_when_web_socket_token_is_missing(self, sns_service_send):
		result = send_notification.execute(WebSocketAction.BROADCAST_TO_ACCOUNT,
										   None, 1234, {'message': 'test'})

		self.assertIsNone(result)
		sns_service_send.assert_not_called()

	@patch('cc_py_commons.config.env.app_config.WEB_SOCKET_AUTH_TOKEN', 'TOKEN')
	def test_valid_request(self, sns_service_send):
		result = send_notification.execute(WebSocketAction.BROADCAST_TO_ACCOUNT,
										   None, 1234, {'message': 'test'})

		self.assertIsNotNone(result)
		sns_service_send.assert_called_once()



import uuid
import datetime
from dataclasses import dataclass, field
from typing import List

@dataclass
class BidHistory:
  id: uuid.UUID
  bid_id: uuid.UUID
  source_type: str
  source_id: int
  amount: int
  carrier_id: uuid.UUID
  pickup_date: datetime.datetime
  delivery_date: datetime.datetime


from marshmallow import fields, EXCLUDE
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema

class FreightHubContactSchema(CamelCaseSchema):
  class Meta:
      unknown = EXCLUDE
        
  contact_name = fields.String()
  contact_phone = fields.String(allow_none=True)
  contact_fax = fields.String(allow_none=True)
  contact_email = fields.String()
  company_name = fields.String()
  user_id = fields.String(allow_none=True)


import unittest

from cc_py_commons.utils.case_conversion import camelize, camel_to_snake_case, to_camel_case, to_snake_case

class TestCaseConversion(unittest.TestCase):

  def test_camelize(self):
    input = 'test_input'
    result = camelize(input)
    self.assertEqual('testInput', result)

  def test_camel_to_snake_case(self):
    input = 'testInput'
    result = camel_to_snake_case(input)
    self.assertEqual('test_input', result)

  def test_to_camel_case(self):
    input = {
      'test_key': 'test_value'
    }
    result = to_camel_case(input)
    self.assertFalse('test_key' in result)
    self.assertTrue('testKey' in result)

  def test_to_snake_case(self):
    input = {
      'testKey': 'test_value'
    }
    result = to_snake_case(input)
    self.assertFalse('testKey' in result)
    self.assertTrue('test_key' in result)



import os

import requests

from cc_py_commons.utils import json_logger

FREIGHT_HUB_URL = os.environ.get('FREIGHT_HUB_URL')
FREIGHT_HUB_TOKEN = os.environ.get("FREIGHT_HUB_TOKEN")


def execute(load_dict, account_id=None):
    load_id = load_dict.get('id')
    url = f"{FREIGHT_HUB_URL}/freight/{load_id}"
    token = f"Bearer {FREIGHT_HUB_TOKEN}"
    headers = {
        "Authorization": token
    }
    try:
        json_logger.debug(account_id, "Updating load on freight-hub", url=url, load_id=load_id)
        response = requests.post(url, json=load_dict, headers=headers, timeout=30)
        if response.status_code not in (200, 201, 204):
            json_logger.warning(account_id, 'FreightHub update returned non-success status',
                                response_status_code=response.status_code,
                                response_text=response.text,
                                load_id=load_id,
                                load_dict=load_dict)
        return response
    except Exception as e:
        json_logger.error(account_id, 'Error calling FreightHub API for update',
            error=str(e), load_id=load_id)
        return None



from dataclasses import dataclass, field

@dataclass
class FreightHubContact:

    contact_name: str = field(default=None)
    contact_phone: str = field(default=None)
    contact_fax: str = field(default=None)
    contact_email: str = field(default=None)
    company_name: str = field(default=None)
    user_id: str = field(default=None)


import uuid
import datetime
from dataclasses import dataclass, field

from cc_py_commons.transactions.equipment_class import EquipmentClass

@dataclass
class Transaction:
    """
    Represents the pricing database loads table. 
    Called Transaction here to clarify the difference between a load that may be active
    and a booked or completed transaction used for pricing.
    """
    client_id: uuid.UUID
    reference_number: str
    pickup_date: datetime.date
    delivery_date: datetime.date
    origin_city: str
    origin_state: str
    origin_postcode: str    
    destination_city: str
    destination_state: str
    destination_postcode: str    
    equipment: str
    equipment_class: EquipmentClass
    all_in_cost: int    
    contact_id: uuid.UUID = field(default=None)
    carrier_id: uuid.UUID = field(default=None)
    booking_date: datetime.date = field(default=None)
    pickup_appointment_window: int = field(default=None)
    num_pickups: int = field(default=None)
    num_dropoffs: int = field(default=None)
    origin_country: str = field(default=None)
    origin_latitude : float = field(default=None)
    origin_longitude : float = field(default=None)
    destination_country: str = field(default=None)
    destination_latitude : float = field(default=None)
    destination_longitude : float = field(default=None)
    miles: int = field(default=None)
    line_haul_cost: int = field(default=None)
    fuel_portion: int = field(default=None)
    price_to_shipper: int = field(default=None)
    weight: int = field(default=None)
    commodity: str = field(default=None)
    comments: str = field(default=None)
    import_stats_id: uuid.UUID = field(default=None)
    #   import_stats = relationship("ImportStats", back_populates="loads", lazy='subquery')
    pickup_appointment_required: bool = field(default=False)
    delivery_appointment_required: bool = field(default=False)
    team_service: bool = field(default=False)
    hand_loading: bool = field(default=False)
    driver_assist_loading: bool = field(default=False)
    driver_assist_unloading: bool = field(default=False)
    drop_trailer: bool = field(default=False)
    hazmat: bool = field(default=False)
    lift_gate_required: bool = field(default=False)
    blanket_wrap: bool = field(default=False)
    white_glove: bool = field(default=False)
    home_delivery: bool = field(default=False)
    etrack_load_stabilization: bool = field(default=False)
    contract: bool = field(default=False)    
    straps_required: bool = field(default=False)
    tarps_required: bool = field(default=False)
    frozen: bool = field(default=False)
    origin_hub_weight : float = field(default=None)
    destination_hub_weight : float = field(default=None)
    calculated_miles: int = field(default=None)
    mileage_mismatch: bool = field(default=False)
    holiday: bool = field(default=False)
    accessorials_included: int = field(default=None)
    client_shipper_id: str = field(default=None)
    client_receiver_id: str = field(default=None)
    load_source_type: str = field(default=None)
    predicted_price: int = field(default=None)
    carrier_code: str = field(default=None)
    customer_code: str = field(default=None)
    customer_id: uuid.UUID = field(default=None)
    customer_name: str = field(default=None)
    account_exec_id: uuid.UUID = field(default=None)
    #   account_exec = relationship("AccountExec", lazy='subquery')
    carrier_rep_id: uuid.UUID = field(default=None)
    #   carrier_rep = relationship("CarrierRep", lazy='subquery')
    excluded: bool = field(default=False)
    lane_id: uuid.UUID = field(default=None)
    multi_stop: int = field(default=1)
    margin: int = field(default=None)
    rate_type: str = field(default=None)
    provider: str = field(default=None)
    target_pay: int = field(default=None)
    max_buy: int = field(default=None)
    # generated by server and returned
    id: uuid.UUID = field(default=None)
    pieces: int = field(default=None)
    max_temperature: float = field(default=None)
    min_temperature: float = field(default=None)
    origin_pallets_required: bool = field(default=False)
    destination_pallets_required: bool = field(default=False)
    origin_location_id: str = field(default=None)
    origin_location_name: str = field(default=None)
    destination_location_id: str = field(default=None)
    destination_location_name: str = field(default=None)
    delivery_open_time: datetime.time = field(default=None)
    delivery_close_time: datetime.time = field(default=None)
    pickup_open_time: datetime.time = field(default=None)
    pickup_close_time: datetime.time = field(default=None)
    freight_hub_load_id: uuid.UUID = field(default=None)
    partial_load: bool = field(default=False)
    linear_feet: int = field(default=None)
    ltl: bool = field(default=False)
    revenue_code: str = field(default=None)
    request_id: uuid.UUID = field(default=None)
    user_id: int = field(default=None)
    po_number: str = field(default=None)



import json
import traceback

from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils import json_logger


def execute(email_message_id, event_type, topic_arn):
	event_type_email_status_map = {
		'open': 'OPENED',
		'click': 'CLICKED',
		'delivered': 'DELIVERED',
		'replied': 'REPLIED'
	}
	if event_type not in event_type_email_status_map:
		json_logger.warning(None, 'Unsupported email event type', email_message_id = email_message_id,
			event_type = event_type, topic_arn = topic_arn)
		return False
	try:
		subject = "handle-procured-carrier-email-event"
		message = {
			"subject": subject,
			"className": "com.cargochief.sdk.carrierhub.notifications.ProcuredCarrierEmailEventNotification",
			"emailStatus": event_type_email_status_map[event_type],
			"emailMessageId": email_message_id
		}
		sns_service = SnsService()
		sns_service.send(topic_arn, message.get('subject'), json.dumps(message))
		return True
	except Exception as e:
		json_logger.error(None, 'Failed to send procured carrier email event',
						  email_message_id=email_message_id, event_type=event_type, topic_arn=topic_arn,
						  error=str(e), stacktrace=traceback.format_exc())
		return False



import json
import traceback

from cc_py_commons.config.env import app_config
from cc_py_commons.config.web_socket_action import WebSocketAction
from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils import json_logger


def execute(action, user_id, account_id, payload):
	if not action or not account_id or not payload:
		json_logger.warning(account_id, 'Missing required parameters')
		return None
	valid_actions = [WebSocketAction.BROADCAST_TO_USER, WebSocketAction.BROADCAST_TO_ACCOUNT]
	if action not in valid_actions:
		json_logger.warning(account_id, f'Invalid action. Should be one of {valid_actions}')
		return None
	web_socket_auth_token = app_config.WEB_SOCKET_AUTH_TOKEN
	if not web_socket_auth_token:
		json_logger.warning(account_id, 'Missing WEB_SOCKET_AUTH_TOKEN')
		return None
	try:
		message = {
			'authKey': web_socket_auth_token,
			'action': action,
			'userId': user_id,
			'accountId': account_id,
			'payload': payload
		}
		sns_service = SnsService()
		sns_service.send(app_config.WEB_SOCKET_SNS_TOPIC_ARN, 'web-socket-message', json.dumps(message))
		successful_message = 'Successfully sent the notification'
		json_logger.info(account_id, successful_message)
		return successful_message
	except Exception as e:
		json_logger.error(account_id, 'Failed to send web socket notification', error=str(e),
						  stacktrace=traceback.format_exc())
	return None



from cc_py_commons.lanes.lane import Lane
from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.carriers.contact_schema import ContactSchema
from cc_py_commons.carriers.account_carrier_map_schema import AccountCarrierMapSchema
from cc_py_commons.lanes.lane_schema import LaneSchema
from cc_py_commons.carriers.carrier import Carrier
class CarrierSchema(CamelCaseSchema):
  class Meta:
    unknown = EXCLUDE

  id = fields.UUID(allow_none=True)
  mc = fields.String(allow_none=True)
  dot = fields.String(allow_none=True)
  customer_code = fields.String(allow_none=True)
  business_name = fields.String()
  doing_business_as = fields.String(allow_none=True)
  address1 = fields.String(allow_none=True)
  city = fields.String(allow_none=True)
  state = fields.String(allow_none=True)
  postcode = fields.String(allow_none=True)
  country = fields.String(allow_none=True)
  latitude = fields.Float(allow_none=True)
  longitude = fields.Float(allow_none=True)
  qualified = fields.Boolean()
  drivers = fields.Integer(allow_none=True)
  active = fields.Boolean()
  active_checked_on = fields.Date(allow_none=True)
  power_units = fields.Integer(allow_none=True)
  credit_score = fields.Integer(allow_none=True)
  days_to_pay = fields.Integer(allow_none=True)
  needs_review = fields.Boolean(allow_none=True)
  is_private = fields.Boolean()
  customer_count = fields.Integer(allow_none=True)
  dispatch_service = fields.Boolean(allow_none=True)
  default_trailer_type = fields.String(allow_none=True)
  carrier_last_fmcsa_update = fields.Date(allow_none=True)
  hazmat = fields.Boolean()
  hm_flag = fields.Boolean()
  mcs_150_date = fields.Date(allow_none=True)
  mcs_150_mileage = fields.Integer(allow_none=True)
  mcs_150_mileage_year = fields.Integer(allow_none=True)
  fmcsa_date_added = fields.Date(allow_none=True)
  fmcsa_oic_state = fields.String(allow_none=True)
  in_network = fields.Boolean()
  last_reviewed = fields.Date(allow_none=True)
  internal_remarks = fields.String(allow_none=True)
  contact_count = fields.Integer(allow_none=True)
  not_reached_count = fields.Integer(allow_none=True)
  not_reached_count_first_updated_at = fields.Date()
  lane_count = fields.Integer(allow_none=True)
  has_teams = fields.Boolean()
  contact = fields.Nested(ContactSchema, allow_none=True)
  account_carrier_maps = fields.List(fields.Nested(AccountCarrierMapSchema))
  lanes = fields.List(fields.Nested(LaneSchema))
  equipment_preferences = fields.List(fields.String)
  no_dispatch = fields.Boolean(allow_none=True)
  contacts = fields.List(fields.Nested(ContactSchema, allow_none=True))
  vetting_authority_display = fields.String(allow_none=True)
  vetting_insurance_display = fields.String(allow_none=True)
  vetting_operations_display = fields.String(allow_none=True)
  vetting_safety_display = fields.String(allow_none=True)
  vetting_overall_display = fields.String(allow_none=True)
  last_vetting_time = fields.Date(allow_none=True)
  vetting_remarks = fields.String(allow_none=True)
  
  @post_load
  def make_carrier(self, data, **kwargs):
      return Carrier(**data)



import requests

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.logger_v2 import logger


def execute(carrier_id):
	carrier = None
	uri = app_config.CARRIER_HUB_URL + '/carrier/' + carrier_id
	request_headers = {
		'Authorization': 'Bearer ' + app_config.CARRIER_HUB_AUTH_TOKEN,
		'Content-Type': 'application/json'
	}
	try:
		response = requests.get(uri, headers=request_headers)
		if response.status_code == 200:
			carrier = response.json()
		else:
			logger.warning(f"Failed to find carrier for id: {carrier_id}")
	except Exception as e:
		logger.error(f'Error: {e} when getting carrier for id: {carrier_id}')
		carrier = None
	return carrier



from marshmallow import Schema, fields

class CamelCaseSchema(Schema):
  """Schema that uses camel-case for its external representation
  and snake-case for its internal representation.
  """

  def on_bind_field(self, field_name, field_obj):
      field_obj.data_key = self.__camelcase(field_obj.data_key or field_name)

  def __camelcase(self, s):
    parts = iter(s.split("_"))
    return next(parts) + "".join(i.title() for i in parts)


import requests

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.logger_v2 import logger

def execute(email_address_list=(), carrier_id_list=(), is_mail_receiver=False, page=0, size=20):
	"""
	Fetch a list of matching contacts from carrier-hub.
	Always returns a list.
	"""
	contacts = []
	if not email_address_list and not carrier_id_list:
		return contacts
	filtered_email_address_list = list(filter(None, email_address_list))
	email_address_string = ','.join(filtered_email_address_list) if email_address_list else ''
	uri = app_config.CARRIER_HUB_URL + '/carrier/-/contact'
	query_params = {
		'page': page,
		'size': size,
		'EMAIL_IDS': email_address_string
	}
	if is_mail_receiver:
		query_params['isMailReciever'] = is_mail_receiver
	if carrier_id_list:
		query_params['CARRIER_IDS'] = ','.join(map(str, carrier_id_list))
	request_headers = {
		'Authorization': 'Bearer ' + app_config.CARRIER_HUB_AUTH_TOKEN,
		'Content-Type': 'application/json'
	}
	try:
		response = requests.get(uri, params=query_params, headers=request_headers)
		if response.status_code == 200:
			contacts = response.json()['content']
			logger.debug(f"For filters: {query_params} number of contacts found: {len(contacts)}")
		else:
			logger.warning(f"Failed to lookup contacts: {response.text}")
	except Exception as e:
		logger.error(f'Error while listing contacts {e}')
		contacts = []

	return contacts



from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.loads.equipment import Equipment

class EquipmentSchema(CamelCaseSchema):
	class Meta:
		unknown = EXCLUDE

	id = fields.UUID()
	name = fields.String()
	active = fields.Boolean(default=True)  # Match the dataclass default

	@post_load
	def make_equipment(self, data, **kwargs):
		return Equipment(**data)
	


import datetime
import uuid
from dataclasses import dataclass, field

@dataclass
class Project:

	id: uuid.UUID
	user_id: int
	account_id: int
	name: str
	assignee_id: int
	commodity: str = field(default=None)
	due_date: datetime.date = field(default=None)
	truck_lane_search_id: uuid.UUID = field(default=None)
	equipment_type: str = field(default=None)
	origin_city: str = field(default=None)
	origin_state: str = field(default=None)
	origin_zip: str = field(default=None)
	dest_city: str = field(default=None)
	dest_state: str = field(default=None)
	dest_zip: str = field(default=None)
	min_rate: int = field(default=None)
	max_rate: int = field(default=None)
	max_buy: int = field(default=None)
	packaging: str = field(default=None)
	trailer_requirements: str = field(default=None)
	shipper_notes: str = field(default=None)
	receiver_notes: str = field(default=None)
	value: int = field(default=None)
	monday: bool = field(default=False)
	tuesday: bool = field(default=False)
	wednesday: bool = field(default=False)
	thursday: bool = field(default=False)
	friday: bool = field(default=False)
	saturday: bool = field(default=False)
	sunday: bool = field(default=False)
	weight: int = field(default=None)
	origin_lat: float = field(default=None)
	origin_lng: float = field(default=None)
	dest_lat: float = field(default=None)
	dest_lng: float = field(default=None)
	load_volume: int = field(default=None)
	rfp_requested_date: datetime.date = field(default=None)
	load_volume_duration: str = field(default=None)
	task_count: int = field(default=None)
	load_capacity: int = field(default=None)
	load_count: int = field(default=None)
	stage_id: uuid.UUID = field(default=None)
	carrier_count: int = field(default=None)
	position: str = field(default=None)
	last_bid: int = field(default=None)
	upload_count: int = field(default=None)
	last_upload: datetime.date = field(default=None)
	best_rate: int = field(default=None)
	match_count: int = field(default=None)
	network_match_count: int = field(default=None)
	probability: int = field(default=None)
	lane_id: uuid.UUID = field(default=None)
	next_pickup_date: datetime.date = field(default=None)
	load_id: uuid.UUID = field(default=None)
	approximate_miles: int = field(default=None)
	require_teams: bool = field(default=False)
	revenue_code: str = field(default=None)
	invite_ungrouped_carriers: bool = field(default=False)
	type: str = field(default=None)
	ungrouped_carriers_delay: int = field(default=None)
	send_to_out_of_network_carriers: bool = field(default=False)
	send_to_out_of_network_carriers_delay: int = field(default=None)
	procured_carrier_id_list: list = field(default=None)
	enabled: bool = field(default=True)
	notify_procured_carriers: bool = field(default=False)
	assignee_is_dispatcher: bool = field(default=False)
	send_to_network_carriers: bool = field(default=False)
	send_to_network_carriers_delay: int = field(default=None)
	deleted: bool = field(default=False)



import traceback

import requests

from cc_py_commons.utils import json_logger
from .constants import C4_API_AUTH_TOKEN, C4_API_URL


def execute(search_request_dict):
    empty_response = { 'data': [], 'nextPage': False}
    if not search_request_dict:
        return empty_response
    url = f"{C4_API_URL}/accounts/search"
    token = f"Bearer {C4_API_AUTH_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(None, 'Searching accounts', url=url, search_request=search_request_dict)
    try:
        response = requests.post(url, json=search_request_dict, headers=headers, timeout=10)
        if response.status_code != 200:
            json_logger.warning(None, 'Request to search accounts failed',
                                response_status_code=response.status_code, response_text=response.text,
                                search_request_dict=search_request_dict)
            return empty_response
        return response.json()
    except requests.exceptions.RequestException as e:
        json_logger.warning(None, 'Request to search accounts failed', error=str(e),
                            search_request_dict=search_request_dict, stacktrace=traceback.format_exc())
        return empty_response
    except ValueError as e:  # JSONDecodeError is a subclass of ValueError
        json_logger.warning(None, 'Failed to parse response JSON', error=str(e),
                            search_request_dict=search_request_dict, stacktrace=traceback.format_exc())
        return empty_response



import uuid
import datetime
from dataclasses import dataclass, field
from typing import List

from cc_py_commons.loads.equipment import Equipment

@dataclass
class Quote:
    load_id: uuid.UUID
    reference_number: str
    customer_id: int
    account_id: int
    origin_city: str
    origin_state: str
    origin_postcode: str
    destination_city: str
    destination_state: str
    destination_postcode: str
    pickup_date: datetime.date
    delivery_date: datetime.date
    status_id: uuid.UUID
    bid_count: int 
    winning_bid_id: uuid.UUID
    booking_id: uuid.UUID
    equipment: List[Equipment]       
    pickup_open_time: datetime.time = field(default=None)
    pickup_close_time: datetime.time = field(default=None)
    pickup_appointment_required: bool = field(default=False)
    delivery_open_time: datetime.time = field(default=None)
    delivery_close_time: datetime.time = field(default=None)
    delivery_appointment_required: bool = field(default=False)
    tracking_number: str = field(default=None)
    length: int = field(default=None)
    width: int = field(default=None)
    height: int = field(default=None)
    weight: int = field(default=None)
    approximate_miles: int = field(default=None)
    declared_price: int = field(default=None)
    declared_value: int = field(default=None)
    special_instructions: str = field(default=None)
    commodity: str = field(default=None)
    min_temperature: float = field(default=None)
    max_temperature: float = field(default=None)
    tarp_size: int = field(default=None)
    team_service_required: bool = field(default=False)
    # generated by server and returned
    id: uuid.UUID = field(default=None)
    # generated by server and returned
    load_number: str = field(default=None)
    max_pay: int = field(default=None)
    stops_count: int = field(default=None)
    stops: list = field(default=None)
    partial_load: bool = field(default=None)
    pieces: int = field(default=None)
    re_opened_at: datetime.date = field(default=None)



import os

import requests

from cc_py_commons.bids.bid_schema import BidSchema
from cc_py_commons.utils import json_logger
from cc_py_commons.utils.bid_utils import parse_dates_in_bid

BOOKING_AGENT_URL = os.environ.get('BOOKING_AGENT_URL')
BOOKING_AGENT_TOKEN = os.environ.get("BOOKING_AGENT_TOKEN")


def execute(account_id, filters_dict):
    url = f"{BOOKING_AGENT_URL}/bid/match"
    token = f"Bearer {BOOKING_AGENT_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(account_id, f"Requesting bids from booking-agent", url=url, filters=filters_dict)
    response = requests.get(url, headers=headers, params=filters_dict)
    bids = []
    if response.status_code == 200:
        bids = response.json()['content']
        for bid in bids:
            parse_dates_in_bid(bid)
    return BidSchema().load(bids, many=True)



import json
from uuid import uuid4
import boto3

class SqsService:
	def send(self, queue_url, message, delaySeconds=0, messageGroupId=None):
		'''
		Sends a message to the specified queue and returns the messageId. 
		The messageId can be used to lookup and delete the message in the queue.
		'''
		sqs = boto3.client('sqs')
		if messageGroupId:
			response = sqs.send_message(
				QueueUrl=queue_url,
				MessageBody=message,
				DelaySeconds=delaySeconds,
				MessageAttributes={},
				MessageGroupId=messageGroupId
			)
		else:
			response = sqs.send_message(
				QueueUrl=queue_url,
				MessageBody=message,
				DelaySeconds=delaySeconds,
				MessageAttributes={}
			)
		return response.get('MessageId')

	def send_batches(self, queue_url, messages):
		'''
		Sends all messages to the specified queue in groups of 10 (the limit of send_batch)
		The messageId can be used to lookup and delete the message in the queue.
		Args:
			queue_url (str): The URL of the Amazon SQS queue
			messages (list): List of messages to be sent

		Returns:
			dict: Results containing 'successful' and 'failed' message lists
		'''
		results = {
			'successful': [],
			'failed': []
		}

		if not messages:
			return results

		sqs = boto3.client('sqs')
		entries = [
			{
				# each message must have a unique id
				'Id': str(uuid4()),
				'MessageBody': json.dumps(message),
			}
			for message in messages
		]

		# SQS can only process 10 messages per batch
		batch_size = 10

		# Split messages into batches of 10
		for i in range(0, len(entries), batch_size):
			batch = entries[i:i + batch_size]

			try:
				response = sqs.send_message_batch(
					QueueUrl=queue_url,
					Entries=batch
				)

				# Track successful and failed messages
				if 'Successful' in response:
					results['successful'].extend(response['Successful'])
				if 'Failed' in response:
					results['failed'].extend(response['Failed'])

			except Exception as e:
				# Mark all messages in failed batch
				failed_batch = [{
					'Id': entry['Id'],
					'Error': str(e),
					'Message': entry['MessageBody']
				} for entry in batch]
				results['failed'].extend(failed_batch)

		return results



import uuid

ACTIVE = uuid.UUID("9bcd4613-831e-4cb4-a76f-9166ce559fa6")
CANCELLED = uuid.UUID("edd2e47a-73a7-406b-85cc-1c339993fdd9")
CONFIRMED = uuid.UUID("adc9769e-d2d7-4931-b373-f0221dcef6c2")
DELIVERED = uuid.UUID("5eca0c1b-1282-49cc-8dc4-8e8eb205e1a1")
DELETED = uuid.UUID("b3f28d9e-828e-459f-ae5a-5db91cabb3e2")
IN_TRANSIT = uuid.UUID("dc9c55e1-e6f8-464e-9a52-b8913ce92ca5")
EXPIRED = uuid.UUID("9f45913b-e4c9-4e0e-a9fb-157fb41e508f")
SELECTED = uuid.UUID("8fa3ae93-b79b-4059-8bfb-e868bc4f858b")
FILLED = uuid.UUID("b3713346-bdfd-4a6a-a1ec-3bbbe8e25836")


import os

import requests

from cc_py_commons.utils import json_logger

LOCATION_API_URL = os.environ.get('LOCATION_API_URL')
LOCATION_API_TOKEN = os.environ.get('LOCATION_API_TOKEN')


def execute(origin_latitude, origin_longitude, destination_latitude, destination_longitude, with_duration=False):
	url = f'{LOCATION_API_URL}/distance?originLatitude={origin_latitude}&originLongitude={origin_longitude}&destinationLatitude={destination_latitude}&destinationLongitude={destination_longitude}'
	if with_duration:
		url = f'{url}&withDuration=true'
	token = f"Bearer {LOCATION_API_TOKEN}"
	headers = {
		"Authorization": token
	}
	json_logger.debug(None, 'Get distance url', url=url)
	response = requests.get(url, headers=headers)
	if response.status_code != 200:
		json_logger.warning(None, 'Request to get distance failed', response_status_code=response.status_code,
							response_text=response.text)
		return None
	if with_duration:
		return response.json()
	else:
		return {'distance': response.json()}



import pendulum

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.redis import distance_db_conn

def pc_miler_limit_reached(logger):
    hourly_cache_key = __get_hourly_key()
    monthly_cache_key = __get_monthly_key()
    return __hourly_limit_reached(hourly_cache_key, logger) or __monthly_limit_reached(monthly_cache_key, logger)

def update_pc_miler_requests_count(logger):
    hourly_cache_key = __get_hourly_key()
    monthly_cache_key = __get_monthly_key()
    __update_hourly_requests_count(hourly_cache_key, logger)
    __update_monthly_requests_count(monthly_cache_key, logger)

def __hourly_limit_reached(hourly_cache_key, logger):
    limit_reached = False
    hourly_requests_count = distance_db_conn.get(hourly_cache_key)
    if hourly_requests_count:
        hourly_requests_count = int(hourly_requests_count)
        limit_reached = (hourly_requests_count >= app_config.PC_MILER_HOURLY_LIMIT)
    if limit_reached:
        logger.warn(f'pc_miler_utils.__hourly_limit_reached - reached hourly limit for {hourly_cache_key} '
                    f'with {hourly_requests_count} hourly calls against permissible {app_config.PC_MILER_HOURLY_LIMIT} calls')
    return limit_reached

def __monthly_limit_reached(monthly_cache_key, logger):
    limit_reached = False
    monthly_requests_count = distance_db_conn.get(monthly_cache_key)
    if monthly_requests_count:
        monthly_requests_count = int(monthly_requests_count)
        limit_reached = (monthly_requests_count >= app_config.PC_MILER_MONTHLY_LIMIT)
    if limit_reached:
        logger.warn(f'pc_miler_utils.__monthly_limit_reached - reached monthly limit for {monthly_cache_key} '
                    f'with {monthly_requests_count} monthly calls against permissible {app_config.PC_MILER_MONTHLY_LIMIT} calls')
    return limit_reached

def __update_hourly_requests_count(hourly_cache_key, logger):
    hourly_requests_count = distance_db_conn.get(hourly_cache_key)
    updated_count = None
    if hourly_requests_count:
        hourly_requests_count = int(hourly_requests_count)
        updated_count = (hourly_requests_count + 1)
    else:
        updated_count = 1
    distance_db_conn.set(hourly_cache_key, updated_count)

def __update_monthly_requests_count(monthly_cache_key, logger):
    monthly_requests_count = distance_db_conn.get(monthly_cache_key)
    updated_count = None
    if monthly_requests_count:
        monthly_requests_count = int(monthly_requests_count)
        updated_count = (monthly_requests_count + 1)
    else:
        updated_count = 1
    distance_db_conn.set(monthly_cache_key, updated_count)

def __get_hourly_key():
    return pendulum.now().set(minute=0,second=0).format('MM-DD-YY HH:mm:ss')

def __get_monthly_key():
    return pendulum.now().format('MM-YY')



from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.loads.equipment_schema import EquipmentSchema
from cc_py_commons.quotes.quote import Quote

class QuoteSchema(CamelCaseSchema):
    class Meta:
        unknown = EXCLUDE

    load_id = fields.UUID()
    reference_number = fields.String()
    customer_id = fields.Integer(allow_none=True)
    account_id = fields.Integer(allow_none=True)
    origin_city = fields.String()
    origin_state = fields.String()
    origin_postcode = fields.String(allow_none=True)
    destination_city = fields.String()
    destination_state = fields.String()
    destination_postcode = fields.String(allow_none=True)
    pickup_date = fields.Date()
    delivery_date = fields.Date()
    status_id = fields.UUID()
    bid_count = fields.Integer(allow_none=True)
    winning_bid_id = fields.UUID(allow_none=True)
    booking_id = fields.UUID(allow_none=True)
    equipment = fields.List(fields.Nested(EquipmentSchema))
    pickup_open_time= fields.Time(allow_none=True)
    pickup_close_time= fields.Time(allow_none=True)
    pickup_appointment_required= fields.Boolean(missing=False, default=False)
    delivery_open_time= fields.Time(allow_none=True)
    delivery_close_time= fields.Time(allow_none=True)
    delivery_appointment_required= fields.Boolean(missing=False, default=False)
    tracking_number = fields.String(allow_none=True)
    length = fields.Integer(allow_none=True)
    width = fields.Integer(allow_none=True)
    height = fields.Integer(allow_none=True)
    weight = fields.Integer(allow_none=True)
    approximate_miles = fields.Integer(allow_none=True)
    declared_price = fields.Integer(allow_none=True)
    declared_value = fields.Integer(allow_none=True)
    special_instructions = fields.String(allow_none=True)
    commodity = fields.String(allow_none=True)
    min_temperature= fields.Float(allow_none=True)
    max_temperature= fields.Float(allow_none=True)
    tarp_size = fields.Integer(allow_none=True)
    team_service_required= fields.Boolean(missing=False, default=False)
    # generated by server and returned
    id = fields.UUID(allow_none=True)
    # generated by server and returned
    load_number = fields.String(allow_none=True)
    max_pay = fields.Integer(allow_none=True)
    stops_count = fields.Integer(allow_none=True)
    stops = fields.List(allow_none=True, cls_or_instance=fields.Raw())
    partial_load = fields.Boolean(allow_none=True)
    pieces = fields.Integer(allow_none=True)
    re_opened_at = fields.Date(allow_none=True)

    @post_load
    def make_quote(self, data, **kwargs):
        return Quote(**data)



import json
import requests

from cc_py_commons.utils.logger_v2 import logger


def execute(channel_webhook_url, message_title, fields_list):
	'''
		sample field: {
			'title': 'Label',
			'value': 'Value'
		}
	'''
	try:
		payload = __payload(message_title, fields_list)
		logger.debug(f'slack.send_slack_message: Sending slack message: {payload} to webhook: {channel_webhook_url}')
		response = requests.post(channel_webhook_url, json.dumps(payload))
		if response.status_code == 200:
			logger.debug(f"Successfully sent message.")
		else:
			logger.warning(f"Failed to send slack message. Response from slack {response.text}")
	except Exception as e:
		logger.error(f'slack.send_slack_message: Failed with error: {e}')

def __payload(message_title, fields_list):
	fields = []
	if fields_list:
		for field in fields_list:
			fields.append({
				'title': field.get('title'),
				'value': field.get('value'),
				'short': True
      })
	payload = {
		'text': f'*{message_title}*',
		'attachments': [{
			'fields': fields
		}]
	}
	return payload



from cc_py_commons.lanes.lane_schema import LaneSchema
from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.lanes.lane_schema import LaneSchema

class CarrierLaneSchema(CamelCaseSchema):
  class Meta:
      unknown = EXCLUDE
  
  carrierId: fields.UUID()
  laneDTO: fields.Nested(LaneSchema)
  hasBackhaulLane = fields.Boolean(allow_none=True)
  isBackhaulLane = fields.Boolean(allow_none=True)
  createBackhaulLanes = fields.Boolean(allow_none=True)
  deleted = fields.Boolean(allow_none=True)
  weeklyFrequency = fields.Integer(allow_none=True)
  runDaily = fields.Boolean(allow_none=True)
  monday = fields.Boolean(allow_none=True)
  tuesday = fields.Boolean(allow_none=True)
  wednesday = fields.Boolean(allow_none=True)
  thursday = fields.Boolean(allow_none=True)
  friday = fields.Boolean(allow_none=True)
  saturday = fields.Boolean(allow_none=True)
  sunday = fields.Boolean(allow_none=True)
  age = fields.Float(allow_none=True)
  rating = fields.Integer(allow_none=True)
  rate = fields.Integer(allow_none=True)
  rateUpdatedAt = fields.Date(allow_none=True)
  fromCarrier = fields.Boolean(allow_none=True)
  toBeReviewed = fields.Boolean(allow_none=True)


import re

def to_camel_case(input_dict):
    """ 
    Converts a dict with keys in snake case to dict with keys
    in camel case
    """
    new_dict = {}
    if isinstance(input_dict, dict):
        for key, value in input_dict.items():
            parsed_key = camelize(key)
            new_dict[parsed_key] = value
    return new_dict

def to_snake_case(input_dict):
    """
    Converts a dict with keys in camel case to dict with keys
    in snake case
    """
    new_dict = {}
    if isinstance(input_dict, dict):
        for key, value in input_dict.items():
            if isinstance(value, dict):
                value = to_snake_case(value) 

            parsed_key = camel_to_snake_case(key)
            new_dict[parsed_key] = value
            
    return new_dict

def camel_to_snake_case(string):
   return re.sub(r'(?<!^)(?=[A-Z])', '_', string).lower()

def camelize(key):
    parts = iter(key.split("_"))
    return next(parts) + "".join(i.title() for i in parts)

def add_undescore_before_numbers(input_dict):
    """
    Handle an issue with the inflection library while converting camel case 
    to underscore
    """
    updated_dict = {}
    for key, value in input_dict.items():
        match = re.search(r"\d", key)
        if match is not None:
            number_position = match.start()
            updated_key = key[:number_position] + '_' + key[number_position:]
            updated_dict[updated_key] = value
        else:
            updated_dict[key] = value

    return updated_dict



import requests

from cc_py_commons.utils.logger_v2 import logger
from .constants import BOOKING_AGENT_TOKEN, BOOKING_AGENT_URL


def execute(bid_id, payload):
  url = f"{BOOKING_AGENT_URL}/quote/-/bid/{bid_id}/counter"
  token = f"Bearer {BOOKING_AGENT_TOKEN}"
  headers = {
    "Authorization": token
  }

  logger.debug(f"Countering bid {bid_id} in booking-agent: {url} with payload: {payload}, {headers}")
  response = requests.post(url, headers=headers, json=payload)

  if response.status_code == 200:
    logger.debug(f"Successfully countered the bid: {bid_id}")
    return response.json()
  else:
    logger.warning(f'Failed to counter bid {response.status_code}:{response.text}')

  return None




def to_celsius(temp_in_fahrenheit):
	temp_in_celsius = None
	if temp_in_fahrenheit:
		temp_in_celsius = round(((temp_in_fahrenheit - 32) * 5) / 9, 2)
	return temp_in_celsius

def to_fahrenheit(temp_in_celsius):
	temp_in_fahrenheit = None
	if temp_in_celsius:
		temp_in_fahrenheit = round(((temp_in_celsius * 9) / 5) + 32, 2)
	return temp_in_fahrenheit



import requests

from cc_py_commons.utils import json_logger
from .constants import BOOKING_AGENT_TOKEN, BOOKING_AGENT_URL


def execute(quote_id, payload, account_id=None):
    url = f"{BOOKING_AGENT_URL}/quote/{quote_id}"
    token = f"Bearer {BOOKING_AGENT_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(account_id, 'Updating quote in booking-agent', url=url, payload=payload)
    response = requests.post(url, headers=headers, json=payload)
    if response.status_code == 200:
        return response.json()
    else:
        json_logger.warning(account_id, 'Failed to update quote', response_status_code=response.status_code,
                       response_text=response.text)
    return None



import googlemaps
import json
import traceback

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.redis import location_db_conn, distance_db_conn, timezone_db_conn
from cc_py_commons.utils import json_logger
from ast import literal_eval

def get_location(city, state, zipcode, logger, reset_cache=False):
  return get_location_with_country(city, state, zipcode, None, logger, reset_cache)

def get_location_with_country(city, state, zipcode, country, logger, reset_cache=False):
  from_cache = False
  postcode = zipcode

  if postcode:
    parts = zipcode.split('-')
    postcode = parts[0] if len(parts) > 0 else zipcode

  data = None
  if not reset_cache:
    data = __get_location_from_cache(city, state, postcode, logger)

  if data:
    if data.get('country') and (not postcode or postcode == data.get('postcode')):
      from_cache = True
    else:
      logger.debug(json.dumps({
        'message': 'Found location in cache that was not a match',
        'city': city,
        'state': state,
        'zipcode': zipcode,
        'postcode': postcode,
        'country': country,
        'cache_key': __get_location_string(city, state, postcode) 
      }))
      data = None
      
  if not data:
    logger.debug(json.dumps({
      'message': 'Could not find location in cache',
      'city': city,
      'state': state,
      'postcode': zipcode,
      'country': country,
      'cache_key': __get_location_string(city, state, postcode) 
    }))
    data = __get_google_location(city, state, postcode, country, logger)

  if not data and postcode:
    from_cache = False
    data = __get_google_location(city=None, state=None, zipcode=postcode, country=None, logger=logger)

  if data and not data.get('postcode'):
    matching_location = __get_matching_location_from_cache(city, state, data.get('lat'), data.get('lng'), logger)
    if matching_location:
      data['postcode'] = matching_location.get('postcode')
    else:
      gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
      response = None
      if zipcode:
          response = gmaps.geocode(zipcode)
      elif data.get('lat') and data.get('lng'):
          response = gmaps.reverse_geocode((data.get('lat'), data.get('lng')))
      post_code_data = __parse_response(response, city)
      data['postcode'] = post_code_data['postcode'] if post_code_data else None

    if data.get('postcode'):
      __cache_location(city, state, zipcode, data, logger)

  if data:
    if not from_cache:
      __cache_location(city, state, zipcode, data, logger)
  else:
    logger.debug(f"geolocate.get_location - Google returned no result for ({city}, {state}, {zipcode}, {country})")

  return data

def get_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude, logger, with_duration=False):
  if not origin_latitude or not origin_longitude or \
    not destination_latitude or not destination_longitude:
    raise Exception("Calculating distance requires both origin and destination lat/lng")

  from_cache = False
  distance_cache_key = __get_distance_cache_key(origin_latitude, origin_longitude, destination_latitude, destination_longitude)
  distance_data_dict = __get_distance_from_cache(distance_cache_key, logger)

  if distance_data_dict:
    from_cache = True
  else:
    json_logger.debug(None, 'Could not find distance from cache', origin_latitude=origin_latitude,
      origin_longitude=origin_longitude, destination_latitude=destination_latitude, destination_longitude=destination_longitude,
      with_duration=with_duration, distance_cache_key=distance_cache_key)

  missing_duration_and_is_required = (with_duration and not distance_data_dict.get('duration'))
  if not distance_data_dict or missing_duration_and_is_required:
      if missing_duration_and_is_required:
        json_logger.debug(None, 'Looking up distance from google because duration is missing in the cached data and it is required in the response',
		  origin_latitude=origin_latitude, origin_longitude=origin_longitude, destination_latitude=destination_latitude,
		  destination_longitude=destination_longitude, with_duration=with_duration, distance_cache_key=distance_cache_key)
      distance_data_dict = __get_google_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude, logger)

  if distance_data_dict:
    if not from_cache or missing_duration_and_is_required:
      json_logger.debug(None, 'Caching the distance', distance_cache_key=distance_cache_key, distance_data_dict=distance_data_dict)
      __cache_distance(distance_cache_key, distance_data_dict, logger)
  else:
    json_logger.debug(None, 'Google returned no result for distance', origin_latitude=origin_latitude,
      origin_longitude=origin_longitude, destination_latitude=destination_latitude, destination_longitude=destination_longitude,
      with_duration=with_duration, distance_cache_key=distance_cache_key)

  if distance_data_dict:
    if with_duration:
      return distance_data_dict
    else:
      return distance_data_dict.get('distance')
  return None

def get_timezone(lat,lng, logger):
  from_cache = False
  timezone = None
  try:
    timezone_cache_key = __get_timezone_cache_key(lat, lng)
    timezone = __get_timezone_from_cache(timezone_cache_key, logger)
    if timezone:
      from_cache = True

    if not timezone:
      gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
      timezone = gmaps.timezone((lat,lng))

    response_status = timezone.get('status')
    if response_status != 'OK':
      msg = ("geolocate.get_timezone - Timezone lookup from google maps failed for "
      	"latitude: {0} and longitude: {1} with status {2}").format(lat, lng, response_status)
      logger.warning(msg)
      timezone = None

    if timezone and not from_cache:
      __cache_timezone(timezone_cache_key, timezone, logger)
  except googlemaps.exceptions.Timeout as e:
    msg = "geolocate.get_timezone - Timezone lookup failed {0}, {1}: {2}".format(lat, lng, e)
    if logger:
      logger.error(msg)
    else:
      print(msg)

  return timezone

def __get_google_distance(origin_latitude, origin_longitude, destination_latitude, destination_longitude, logger):
  distance_data = None
  try:
    gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
    origins = [f'{origin_latitude} {origin_longitude}']
    destinations = [f'{destination_latitude} {destination_longitude}']
    response = gmaps.distance_matrix(origins,
                                     destinations,
                                     mode="driving",
                                     avoid='ferries',
                                     units="imperial")
    if response and response['rows'][0]['elements'][0].get('distance', None):
      distance_in_meters = response['rows'][0]['elements'][0]['distance']['value']
      distance = distance_in_meters / 1609
      duration_seconds = response['rows'][0]['elements'][0]['duration']['value']
      duration_minutes = duration_seconds / 60
      distance_data = {
        'distance': distance,
        'duration': duration_minutes
      }
  except Exception as e:
    logger.error("geolocate.__get_google_distance: Error while getting distance from google", e)
    distance_data = None
  return distance_data

def __get_google_location(city, state, zipcode, country, logger):
  try:
    if (not city or not state) and not zipcode:
      return None
    loc_str = ''
    data = None

    if city:
      loc_str = city

    if state:
      if len(loc_str) > 0:
        loc_str = loc_str + ', '

      loc_str = loc_str + state

    if zipcode:
      if len(loc_str) > 0:
        loc_str = loc_str + ' '

      loc_str = loc_str + zipcode

    if country:
      if len(loc_str) > 0:
        loc_str = loc_str + ', '

      loc_str = loc_str + country

    gmaps = googlemaps.Client(key=app_config.GOOGLE_API_KEY)
    response = gmaps.geocode(loc_str)
    
    if response:
      response = [results for results in response if 'country' not in results.get('types')]

      if len(response) > 1:
        logger.warning(f"Google returned multiple matches for {loc_str}. Selecting a single address.")
        selected_address = None
        if city and state:
          selected_address = __select_address(city=city, state=state, address_list=response)
        if not selected_address:
          logger.warning(f"Cannot select a single address from multiple for {loc_str}.")
          return None
        response = [selected_address]
      return __parse_response(response, city)
  except googlemaps.exceptions.Timeout as e:
    msg = "geolocate.__get_google_location - Location lookup timed out {0}, {1}, {2}: {3}".format(city, state, zipcode, e)

    if logger:
      logger.error(msg)
    else:
      print(msg)
  except googlemaps.exceptions.HTTPError as e:
    msg = "geolocate.__get_google_location - Location lookup failed {0}, {1}, {2}: {3}".format(city, state, zipcode, e)

    if logger:
      logger.error(msg)
    else:
      print(msg)

  return None

def __parse_response(response, city):
    if not response or len(response) == 0:
        return None
    ''' 
      Sample response of Google Maps API for address: Cincinati, OH, 45204
      {
          "address_components": [
              {
                  "long_name": "45204",
                  "short_name": "45204",
                  "types": [
                      "postal_code"
                  ]
              },
              {
                  "long_name": "Cincinnati",
                  "short_name": "Cincinnati",
                  "types": [
                      "locality",
                      "political"
                  ]
              },
              {
                  "long_name": "Hamilton County",
                  "short_name": "Hamilton County",
                  "types": [
                      "administrative_area_level_2",
                      "political"
                  ]
              },
              {
                  "long_name": "Ohio",
                  "short_name": "OH",
                  "types": [
                      "administrative_area_level_1",
                      "political"
                  ]
              },
              {
                  "long_name": "United States",
                  "short_name": "US",
                  "types": [
                      "country",
                      "political"
                  ]
              }
          ],
          "formatted_address": "Cincinnati, OH 45204, USA",
          "geometry": {
              "bounds": {
                  "northeast": {
                      "lat": 39.1251898,
                      "lng": -84.539683
                  },
                  "southwest": {
                      "lat": 39.073286,
                      "lng": -84.62870989999999
                  }
              },
              "location": {
                  "lat": 39.0930395,
                  "lng": -84.56676650000001
              },
              "location_type": "APPROXIMATE",
              "viewport": {
                  "northeast": {
                      "lat": 39.1251898,
                      "lng": -84.539683
                  },
                  "southwest": {
                      "lat": 39.073286,
                      "lng": -84.62870989999999
                  }
              }
          },
          "place_id": "ChIJjYEP0hK2QYgRTJoYykoQeqw",
          "postcode_localities": [
              "Cincinnati",
              "Queen City Square"
          ],
          "types": [
              "postal_code"
          ]
      }
    '''
    components = response[0]['address_components']
    types = response[0]['types']
    location = response[0]['geometry']['location']
    postcode_localities = response[0].get('postcode_localities', None)

    data = {'city': None, 'state': None, 'postcode': None}
    data['lat'] = location['lat']
    data['lng'] = location['lng']

    # when checking the response for a zipcode verify the city name is in the list of localities
    if 'postal_code' in types and postcode_localities and city:
        matches = [l for l in components if (
            'postal_localities' in l.get('types') and
            (l.short_name.lower() == city.lower() or l.long_name.lower() == city.lower())
        )]
        city_present_in_postcode_localities = (city in postcode_localities)

        if not matches and not city_present_in_postcode_localities:
            return None

    for component in components:
        if 'locality' in component['types']:
            data['city'] = component['short_name']
        elif 'administrative_area_level_1' in component['types']:
            data['state'] = component['short_name']
        elif 'postcode' in component['types'] or 'postal_code' in component['types']:
            data['postcode'] = component['short_name']
        elif 'country' in component['types']:
            data['country'] = component['short_name']

    return data

def __cache_location(city, state, zipcode, location_data, logger):
    location_string = __get_location_string(city, state, zipcode)
    location_db_conn.set(location_string, str(location_data))

def __get_location_from_cache(city, state, zipcode, logger):
  location = None

  try:
    cache_key = __get_location_string(city, state, zipcode) 
    location = location_db_conn.get(cache_key)

    if location:
      location = literal_eval(location.decode('utf-8'))
  except Exception as e:
    logger.warn("geolocate.__get_location_from_cache: Error while getting location from cache", e)
    location = None

  return location

def __get_location_string(city, state, zipcode):
  location_string = ''
  if city:
    location_string += ('_'.join(city.split(' ')))
  if state:
    location_string += f'_{state}'
  if zipcode:
    location_string += f'_{zipcode}'

  return location_string.lower()

def __cache_distance(distance_cache_key, distance_data, logger):
  distance_db_conn.set(distance_cache_key, str(distance_data))

def __get_distance_from_cache(distance_cache_key, logger):
  distance_data = {}
  try:
    cached_data = distance_db_conn.get(distance_cache_key)
    if cached_data:
      if 'duration' in str(cached_data):
        distance_data = literal_eval(cached_data.decode('utf-8'))
      else:
        distance_data['distance'] = float(cached_data)
  except Exception as e:
    json_logger.warning(None, 'Error while getting distance from cache',
    	error=str(e), stacktrace=traceback.format_exc())
    distance_data = {}
  return distance_data

def __get_distance_cache_key(origin_latitude, origin_longitude, destination_latitude, destination_longitude):
  return f'{__truncate(origin_latitude)},{__truncate(origin_longitude)}->{__truncate(destination_latitude)},{__truncate(destination_longitude)}'

def __truncate(number):
  return int(number * 1000000) / 1000000

def __cache_timezone(timezone_cache_key, timezone, logger):
  timezone_db_conn.set(timezone_cache_key, json.dumps(timezone))

def __get_timezone_from_cache(timezone_cache_key, logger):
  timezone = None
  try:
    timezone = timezone_db_conn.get(timezone_cache_key)
    if timezone:
      timezone = literal_eval(timezone.decode('utf-8'))
  except Exception as e:
    logger.warn("geolocate.__get_timezone_from_cache - Error while getting timezone from cache", e)
    timezone = None
  return timezone

def __get_timezone_cache_key(lat,lng):
  return f'{lat},{lng}'

def __select_address(city, state, address_list):
  city_and_state_list = []
  # Parse city and state from multiple addresses
  for address in address_list:
    address_components = address.get('address_components', [])
    city_and_state = {'city': None, 'state': None, 'address': address}
    for address_component in address_components:
      types = address_component.get('types', [])
      if not city_and_state['city'] and ('locality' in types or 'administrative_area_level_3' in types):
        city_and_state['city'] = address_component.get('long_name')
      elif not city_and_state['state'] and ('country' in types or 'administrative_area_level_1' in types):
        city_and_state['state'] = address_component.get('short_name')

      if city_and_state['city'] and city_and_state['state']:
        city_and_state_list.append(city_and_state)
        break

  # Select the address matching the input city and state
  for parsed_city_and_state in city_and_state_list:
    if parsed_city_and_state['city'].strip().lower() == city.strip().lower() \
      and parsed_city_and_state['state'].strip().lower() == state.strip().lower():
      return parsed_city_and_state['address']
  return None

def __get_matching_location_from_cache(city, state, lat, lng, logger):
    matching_location = None
    try:
        if city and state and lat and lng:
            key = __get_location_string(city, state, None)
            pattern = f'{key}*'
            matching_keys = location_db_conn.keys(pattern)
            if matching_keys:
                for matching_key in matching_keys:
                    location = literal_eval(location_db_conn.get(matching_key).decode('utf-8'))
                    if location.get('lat') == lat and location.get('lng') == lng and location.get('postcode'):
                        matching_location = location
                        break
    except Exception as e:
        logger.warn("geolocate.__get_matching_location_from_cache: Failed: ", e)
    return matching_location



from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.carriers.account_carrier_map import AccountCarrierMap
class AccountCarrierMapSchema(CamelCaseSchema):
  class Meta:
    unknown = EXCLUDE

  account_id = fields.Integer()
  carrier_id = fields.UUID()
  status = fields.String()
  customer_code = fields.String(allow_none=True, missing=None)
  no_dispatch = fields.Boolean(allow_none=True, missing=None)
  
  @post_load
  def make_account_carrier_map(self, data, **kwargs):
      return AccountCarrierMap(**data)  


import os

import requests
from dateutil.parser import parse

from cc_py_commons.quotes.quote_schema import QuoteSchema
from cc_py_commons.utils import json_logger

BOOKING_AGENT_URL = os.environ.get('BOOKING_AGENT_URL')
BOOKING_AGENT_TOKEN = os.environ.get("BOOKING_AGENT_TOKEN")


def execute(account_id, filters_dict):
    url = f"{BOOKING_AGENT_URL}/quote"
    token = f"Bearer {BOOKING_AGENT_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(account_id, f"Requesting quotes from booking-agent", url=url, filters=filters_dict)
    response = requests.get(url, headers=headers, params=filters_dict)
    quotes = []
    if response.status_code == 200:
        quotes = response.json()['content']
        for quote in quotes:
            quote['pickupDate'] = parse(quote['pickupDate']).strftime('%Y-%m-%d')
            quote['deliveryDate'] = parse(quote['deliveryDate']).strftime('%Y-%m-%d')
            if quote.get('reOpenedAt'):
                quote['reOpenedAt'] = parse(quote['reOpenedAt']).strftime('%Y-%m-%d')
        quotes = QuoteSchema().load(quotes, many=True)
    return quotes



class C4AccountSettings:
	GREENSCREENS_RATE_IN_RFP = 'GREENSCREENS_RATE_IN_RFP'
	CAPACITY_DATA_IN_RFP = 'CAPACITY_DATA_IN_RFP'
	SEARCH_TRUCK_ORIGIN_DEADHEAD_LIMIT = 'SEARCH_TRUCK_ORIGIN_DEADHEAD_LIMIT'
	SEARCH_TRUCK_DESTINATION_DEADHEAD_LIMIT = 'SEARCH_TRUCK_DESTINATION_DEADHEAD_LIMIT'
	SEARCH_TRUCK_RESULTS_COUNT = 'SEARCH_TRUCK_RESULTS_COUNT'
	INCLUDE_DISPATCHER_IN_BOOK_NOW_OFFER = 'INCLUDE_DISPATCHER_IN_BOOK_NOW_OFFER'
	AUTO_INVITE_CARRIERS = 'AUTO_INVITE_CARRIERS'



import logging
import json

from cc_py_commons.utils.logger_v2 import logger

def log(level, account_id, message, **kwargs):
  ###
  # Takes log level (ERROR, WARN, DEBUG, INFO), account_id a message, and any number of named arguments 
  # and generates a JSON log entry using the V2 logger.
	# 
	# Dictionaries containing UUID attributes will cause an error
  ###
  obj = {
    'account_id': account_id,
    'message': message
  }
  obj.update(kwargs)  
  logger.log(level, json.dumps(obj))
  
def error(account_id, message, **kwargs):
  log(logging.ERROR, account_id, message, **kwargs, exec_info=1)
  
def warning(account_id, message, **kwargs):
  log(logging.WARNING, account_id, message, **kwargs)

def info(account_id, message, **kwargs):
  log(logging.INFO, account_id, message, **kwargs)
  
def debug(account_id, message, **kwargs):
  log(logging.DEBUG, account_id, message, **kwargs)	


import os
import requests

MERCURY_URL = os.environ.get("MERCURY_URL")
MERCURY_TOKEN = os.environ.get("MERCURY_TOKEN")

from cc_py_commons.utils.logger_v2 import logger


def execute(c4_account_id):
  url = f"{MERCURY_URL}/clients/account-folder/{c4_account_id}"
  token = f"Bearer {MERCURY_TOKEN}"
  headers = {
    "Authorization": token
  }
  response = requests.post(url, headers=headers)
  if response.status_code not in [200, 201]:
    logger.error(f"Failed to create account folder for account {c4_account_id} - {response.status_code}:{response.text}")
    return None
  else:
    return response.json()



import traceback

import requests

from cc_py_commons.config.env import app_config
from cc_py_commons.projects.project_schema import ProjectSchema
from cc_py_commons.utils import json_logger


def execute(project_id, account_id=None):
	project = None
	uri = app_config.CARRIER_HUB_URL + '/projects/id/' + project_id
	request_headers = {
		'Authorization': 'Bearer ' + app_config.CARRIER_HUB_AUTH_TOKEN,
		'Content-Type': 'application/json'
	}
	try:
		response = requests.get(uri, headers=request_headers)
		if response.status_code == 200:
			project_json = response.json()
			project = ProjectSchema().load(project_json)
		else:
			json_logger.warning(account_id, 'Failed to find project', project_id=project_id,
								response_status_code=response.status_code, response_text=response.text)
	except Exception as e:
		json_logger.error(account_id, 'Exception when getting project',
						  project_id=project_id, error=str(e), stacktrace=traceback.format_exc())
		project = None
	return project



import os

import requests

from cc_py_commons.utils import json_logger

MERCURY_URL = os.environ.get("MERCURY_URL")
MERCURY_TOKEN = os.environ.get("MERCURY_TOKEN")


def execute(mc=None, dot=None):
    if not MERCURY_URL or not MERCURY_TOKEN:
        json_logger.error(None, 'MERCURY_URL or MERCURY_TOKEN environment variables not set')
        return None
    if not mc and not dot:
        return None
    params = {}
    if mc:
        params['mc'] = mc
    if dot:
        params['dot'] = dot
    url = f"{MERCURY_URL}/fmcsa/carrier"
    token = f"Bearer {MERCURY_TOKEN}"
    headers = {
        "Authorization": token
    }
    try:
        response = requests.get(url, params=params, headers=headers, timeout=(5, 30))
    except requests.RequestException as e:
        json_logger.error(None, 'Request to FMCSA failed', mc=mc, dot=dot, error=str(e))
        return None

    if response.status_code != 200:
        json_logger.error(None, 'Could not search from FMCSA', mc=mc, dot=dot,
                          response_status_code=response.status_code, response_text=response.text)
        return None
    try:
        return response.json()
    except ValueError as e:
        json_logger.error(None, 'Invalid JSON response from FMCSA', mc=mc, dot=dot, error=str(e))
        return None



import datetime
import uuid
from dataclasses import dataclass, field

@dataclass
class AccountCarrierMap:
  account_id: int
  carrier_id: uuid.UUID
  status: str
  customer_code: str = field(default=None)
  warmup_email_sent_at: datetime.date = field(default=None)
  no_dispatch: bool = field(default=False)


from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema

class BidHistorySchema(CamelCaseSchema):
  class Meta:
    unknown = EXCLUDE

  id = fields.UUID()
  bid_id = fields.UUID()
  source_type = fields.String(allow_none=True)
  source_id = fields.Integer(allow_none=True)
  amount = fields.Integer()
  carrier_id = fields.UUID(allow_none=True)
  pickup_date = fields.Date()
  delivery_date = fields.Date()



def build_full_name(first_name: str = None, last_name: str = None) -> str:
    """
    Build a full name from first and last name components.

    Args:
        first_name: The first name (optional)
        last_name: The last name (optional)

    Returns:
        A formatted full name string with proper spacing
    """
    names = [name.strip() for name in [first_name, last_name] if name and name.strip()]
    return ' '.join(names)



import datetime
import uuid
from dataclasses import dataclass, field
from typing import List, Dict

from cc_py_commons.carriers.contact import Contact
from cc_py_commons.carriers.account_carrier_map import AccountCarrierMap
from cc_py_commons.lanes.lane import Lane

@dataclass
class Carrier:
      
  mc: str = field(default=None)
  dot: str = field(default=None)  
  qualified: bool = field(default=None)
  no_dispatch: bool = field(default=None)
  active: bool = field(default=False)
  city: str = field(default=None)
  state: str = field(default=None)
  postcode: str = field(default=None)
  country: str = field(default=None)
  latitude: float = field(default=None)
  longitude: float = field(default=None)
  id: uuid.UUID = field(default=None)
  customer_code: str = field(default=None)
  business_name: str = field(default=None)
  doing_business_as: str = field(default=None)
  address1: str = field(default=None)
  drivers: int = field(default=None)
  active_checked_on: datetime.date = field(default_factory=lambda: datetime.date.today())
  power_units: int = field(default=None)
  credit_score: int = field(default=None)
  days_to_pay: int = field(default=None)
  needs_review: bool = field(default=False)
  is_private: bool = field(default=False)
  dispatch_service: bool = field(default=False)
  default_trailer_type: str = field(default=None)
  carrier_last_fmcsa_update: datetime.datetime = field(default=None)
  hazmat: bool = field(default=False)
  hm_flag: bool = field(default=False)
  hazmat_expiration_date: datetime.datetime = field(default=None)
  mcs_150_date: datetime.date = field(default=None)
  mcs_150_mileage: int = field(default=None)
  mcs_150_mileage_year: int = field(default=None)
  fmcsa_date_added: datetime.datetime = field(default=None)
  fmcsa_oic_state: str = field(default=None)
  fmcsa_safety_registration_date: datetime.datetime = field(default=None)
  fmcsa_authority_registration_date: datetime.datetime = field(default=None)
  in_network: bool = field(default=False)
  last_reviewed: datetime.date = field(default=None)
  has_teams: bool = field(default=False)
  cargo_insurance_amount: float = field(default=None)
  cargo_insurance_renewal_date: datetime.datetime = field(default=None)
  liability_expiration_date: datetime.datetime = field(default=None)
  workmans_comp_date: datetime.datetime = field(default=None)
  safety_rating_date: datetime.datetime = field(default=None)
  safety_review_date: datetime.datetime = field(default=None)
  customer_count: int = field(default=0)
  contact_count: int = field(default=0)
  not_reached_count: int = field(default=0)
  not_reached_count_first_updated_at: datetime.datetime = field(default=None)
  lane_count: int = field(default=0)
  internal_remarks: str = field(default=None)
  contact: Contact = field(default=None)
  account_carrier_maps: List[AccountCarrierMap] = field(default_factory=lambda: [])
  lanes: List[Lane] = field(default_factory=lambda: [])
  carrier_manager_id: str = field(default=None)
  contacts: List[Contact] = field(default=None)
  vetting_authority_display: str = field(default=None)
  vetting_insurance_display: str = field(default=None)
  vetting_operations_display: str = field(default=None)
  vetting_safety_display: str = field(default=None)
  vetting_overall_display: str = field(default=None)
  last_vetting_time: datetime.datetime = field(default=None)
  vetting_remarks: str = field(default=None)



import json
import traceback

from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils import json_logger


class BookingAssistantFlowNotification:

	def __init__(self, app_config):
		self._app_config = app_config

	def send(self, user_id, load, params, capacity_search_equipments, request_id, auto_invite_details=None):
		'''
	  Sends and event to the Booking Assistant Flow SQS queue specified in the config.
	  Returns the messageId of the enqueued message for later retrieval.
	  '''
		c4_account_id = load.get('accountId')
		subject = self._app_config.BOOKING_ASSISTANT_SNS_SUBJECT
		event = {
			'userId': user_id,
			'loadDTO': load,
			'payload': params,
			'capacitySearchEquipments': capacity_search_equipments,
			'requestId': request_id,
			'subject': f'{subject}',
			'className': f'{self._app_config.BOOKING_ASSISTANT_SNS_CLASS_NAME}',
			'autoInviteDetails': auto_invite_details
		}
		json_logger.debug(c4_account_id, 'Sending booking assistant flow SNS notification',
						  topic=self._app_config.BOOKING_ASSISTANT_SNS_TOPIC_ARN, event=event)
		try:
			sns_service = SnsService()
			return sns_service.send(self._app_config.BOOKING_ASSISTANT_SNS_TOPIC_ARN, subject, json.dumps(event))
		except Exception as e:
			json_logger.warning(c4_account_id, 'Failed to send BookingAssistantFlowNotification',
								error=str(e), stacktrace=traceback.format_exc())



from dataclasses import MISSING
from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.carriers.contact import Contact
class ContactSchema(CamelCaseSchema):
  class Meta:
    unknown = EXCLUDE

  id = fields.UUID(allow_none=True)
  carrier_id = fields.UUID(allow_none=True)
  first_name = fields.String(allow_none=True)
  last_name = fields.String(allow_none=True)
  phone = fields.String(allow_none=True)
  email_address = fields.Email()
  mobile = fields.String(allow_none=True)
  primary_contact = fields.Boolean(allow_none=True, missing=False)
  is_mail_reciever: bool = fields.Boolean(allow_none=True, missing=False)

  @post_load
  def make_bid(self, data, **kwargs):
      return Contact(**data)  


from marshmallow import fields, EXCLUDE, post_load

from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.projects.project import Project


class ProjectSchema(CamelCaseSchema):
	class Meta:
		unknown = EXCLUDE

	id = fields.UUID(allow_none=True)
	user_id = fields.Integer(allow_none=True)
	account_id = fields.Integer(allow_none=True)
	name = fields.String(allow_none=True)
	assignee_id = fields.Integer(allow_none=True)
	commodity = fields.String(allow_none=True)
	due_date = fields.Date(allow_none=True)
	truck_lane_search_id = fields.UUID(allow_none=True)
	equipment_type = fields.String(allow_none=True)
	origin_city = fields.String(allow_none=True)
	origin_state = fields.String(allow_none=True)
	origin_zip = fields.String(allow_none=True)
	dest_city = fields.String(allow_none=True)
	dest_state = fields.String(allow_none=True)
	dest_zip = fields.String(allow_none=True)
	min_rate = fields.Integer(allow_none=True)
	max_rate = fields.Integer(allow_none=True)
	max_buy = fields.Integer(allow_none=True)
	packaging = fields.String(allow_none=True)
	trailer_requirements = fields.String(allow_none=True)
	shipper_notes = fields.String(allow_none=True)
	receiver_notes = fields.String(allow_none=True)
	value = fields.Integer(allow_none=True)
	monday = fields.Boolean(allow_none=True)
	tuesday = fields.Boolean(allow_none=True)
	wednesday = fields.Boolean(allow_none=True)
	thursday = fields.Boolean(allow_none=True)
	friday = fields.Boolean(allow_none=True)
	saturday = fields.Boolean(allow_none=True)
	sunday = fields.Boolean(allow_none=True)
	weight = fields.Integer(allow_none=True)
	origin_lat = fields.Float(allow_none=True)
	origin_lng = fields.Float(allow_none=True)
	dest_lat = fields.Float(allow_none=True)
	dest_lng = fields.Float(allow_none=True)
	load_volume = fields.Integer(allow_none=True)
	rfp_requested_date = fields.Date(allow_none=True)
	load_volume_duration = fields.String(allow_none=True)
	task_count = fields.Integer(allow_none=True)
	load_capacity = fields.Integer(allow_none=True)
	load_count = fields.Integer(allow_none=True)
	stage_id = fields.UUID(allow_none=True)
	carrier_count = fields.Integer(allow_none=True)
	position = fields.String(allow_none=True)
	last_bid = fields.Integer(allow_none=True)
	upload_count = fields.Integer(allow_none=True)
	last_upload = fields.Date(allow_none=True)
	best_rate = fields.Integer(allow_none=True)
	match_count = fields.Integer(allow_none=True)
	network_match_count = fields.Integer(allow_none=True)
	probability = fields.Integer(allow_none=True)
	lane_id = fields.UUID(allow_none=True)
	next_pickup_date = fields.Date(allow_none=True)
	load_id = fields.UUID(allow_none=True)
	approximate_miles = fields.Integer(allow_none=True)
	require_teams = fields.Boolean(allow_none=True)
	revenue_code = fields.String(allow_none=True)
	invite_ungrouped_carriers = fields.Boolean(allow_none=True)
	type = fields.String(allow_none=True)
	ungrouped_carriers_delay = fields.Integer(allow_none=True)
	send_to_out_of_network_carriers = fields.Boolean(allow_none=True)
	send_to_out_of_network_carriers_delay = fields.Integer(allow_none=True)
	procured_carrier_id_list = fields.List(fields.UUID, allow_none=True)
	enabled = fields.Boolean(allow_none=True)
	notify_procured_carriers = fields.Boolean(allow_none=True)
	assignee_is_dispatcher = fields.Boolean(allow_none=True)
	send_to_network_carriers = fields.Boolean(allow_none=True)
	send_to_network_carriers_delay = fields.Integer(allow_none=True)
	deleted = fields.Boolean(allow_none=True)

	@post_load
	def make_project(self, data, **kwargs):
		return Project(**data)



import requests

from cc_py_commons.utils.logger_v2 import logger
from .constants import BOOKING_AGENT_TOKEN, BOOKING_AGENT_URL


def execute(bid_id, payload):
	url = f"{BOOKING_AGENT_URL}/quote/-/bid/{bid_id}"
	token = f"Bearer {BOOKING_AGENT_TOKEN}"
	headers = {
		"Authorization": token
	}

	logger.debug(f"Updating bid {bid_id} in booking-agent: {url} with payload: {payload}, {headers}")
	response = requests.post(url, headers=headers, json=payload)

	if response.status_code == 200:
		logger.debug(f"Successfully updated the bid: {bid_id}")
		return response.json()
	else:
		logger.warning(f'Failed to update bid {response.status_code}:{response.text}')

	return None



import uuid
import datetime
from dataclasses import dataclass, field
from typing import List

from cc_py_commons.bids.bid_history import BidHistory

@dataclass
class Bid:
  id: uuid.UUID
  quote_id: uuid.UUID
  carrier_id: uuid.UUID
  receipt_id: str
  amount: int
  estimated_days: int
  notes: str
  match_score: float
  status_id: uuid.UUID
  pickup_date: datetime.datetime
  delivery_date: datetime.datetime
  decline_reason: str
  bid_histories: List[BidHistory]
  porus_truck_id: uuid.UUID 
  truck_lane_id: uuid.UUID
  distance: float
  origin_city: str
  origin_state: str
  dest_city: str
  dest_state: str
  origin_zip: str = field(default=None)
  origin_deadhead: float = field(default=None)
  dest_zip: str = field(default=None)
  invite_emailed_at: datetime.datetime = field(default=None)



import requests

from cc_py_commons.bids.bid_schema import BidSchema
from cc_py_commons.utils import json_logger
from cc_py_commons.utils.bid_utils import parse_dates_in_bid
from .constants import BOOKING_AGENT_URL, BOOKING_AGENT_TOKEN


def execute(account_id, bid_id):
    url = f"{BOOKING_AGENT_URL}/quote/-/bid/{bid_id}"
    token = f"Bearer {BOOKING_AGENT_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(account_id, f'Requesting bid from booking-agent', url=url)
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        bid_dict = response.json()
        parse_dates_in_bid(bid_dict)
        return BidSchema().load(bid_dict)
    else:
        json_logger.warning(account_id, 'Could not find bid from booking-agent',
                            response_status_code=response.status_code, response_text=response.text)
    return None



import traceback

import requests

from cc_py_commons.utils import json_logger
from .constants import C4_API_AUTH_TOKEN, C4_API_URL


def execute(account_id, update_request_dict):
    if not update_request_dict:
        return None
    url = f"{C4_API_URL}/accounts"
    token = f"Bearer {C4_API_AUTH_TOKEN}"
    headers = {
        "Authorization": token
    }
    json_logger.debug(account_id, 'Updating account', url=url, update_request=update_request_dict)
    try:
        response = requests.patch(url, json=update_request_dict, headers=headers, timeout=10)
        if response.status_code != 200:
            json_logger.warning(account_id, 'Request to update account failed',
                                response_status_code=response.status_code, response_text=response.text,
                                update_request_dict=update_request_dict)
            return None
        return response.json()
    except requests.exceptions.RequestException as e:
        json_logger.warning(account_id, 'Request to update account failed', error=str(e),
                            update_request_dict=update_request_dict, stacktrace=traceback.format_exc())
        return None
    except ValueError as e:  # JSONDecodeError is a subclass of ValueError
        json_logger.warning(account_id, 'Failed to parse response JSON', error=str(e),
                            update_request_dict=update_request_dict, stacktrace=traceback.format_exc())
        return None



import requests

from cc_py_commons.utils import json_logger
from .constants import BOOKING_AGENT_TOKEN, BOOKING_AGENT_URL


def execute(load_id):
	url = f"{BOOKING_AGENT_URL}/quote/by/loadId/{load_id}"
	booking_agent_headers = {
		"Authorization": f"Bearer {BOOKING_AGENT_TOKEN}",
		"Content-Type": "application/json"
	}
	response = requests.get(url, headers=booking_agent_headers)
	if response.status_code == 200:
		return response.json()
	else:
		json_logger.warning(None, 'Failed to get quote from Booking Agent',
							status_code=response.status_code, load_id=str(load_id))
		return None



from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.lanes.lane import Lane

class LaneSchema(CamelCaseSchema):
  class Meta:
      unknown = EXCLUDE

  trailer_type = fields.String()
  origin_city = fields.String()
  origin_state = fields.String()
  dest_city = fields.String()
  dest_state = fields.String()  
  origin_zip = fields.String(allow_none=True)
  origin_lat = fields.Float(allow_none=True)
  origin_lng = fields.Float(allow_none=True)
  origin_deadhead = fields.Float(allow_none=True)
  dest_zip = fields.String(allow_none=True)
  destination_states = fields.String(allow_none=True)
  dest_lat = fields.Float(allow_none=True)
  dest_lng = fields.Float(allow_none=True)
  destination_deadhead = fields.Float(allow_none=True)
  priced_loads = fields.Integer(allow_none=True, missing=0)
  # generated by server
  id = fields.UUID(allow_none=True)


###
# Singleton logger that removes issues with the logger being loaded multiple times and duplicating log entries
# Logs to stdout unless the ENV variable LOG_DESTINATION is present.
# Currently LOG_DESTINATION only support PAPERTRAIL
#
# Using the json_logger wrapper is preferred.
###
import os
import logging
import socket

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.datadog_logger import DataDogLogHandler

PAPERTRAIL = 'PAPERTRAIL'
DATADOG = 'DATADOG'

class ContextFilter(logging.Filter):
	hostname = socket.gethostname()

	def filter(self, record):
		record.hostname = ContextFilter.hostname
		return True

class SingletonLogger:
	"""
	Creates a single instance of the logger with the given name.
	Looks for a LOG_DESTINATION ENV variable to determine where to send logs. 
	Logs to Standard Out by default. 
	Standard Out is automatically captured by DataDog and works for Lambdas.
	"""
	_instance = None

	@staticmethod
	def getInstance():
		if SingletonLogger._instance == None:
			SingletonLogger()

		return SingletonLogger._instance

	def __init__(self):
		if SingletonLogger._instance != None:
			raise Exception("This class is a singleton!")
		else:
			SingletonLogger._instance = self._config_logger()

	def _config_logger(self):
		logger = logging.getLogger(app_config.LOG_APP_NAME)
		logger.setLevel(app_config.LOG_LEVEL)
		contextFilter = ContextFilter()
		logger.addFilter(contextFilter)
		formatter = logging.Formatter('%(levelname)s %(message)s', datefmt='%Y-%m-%d %I:%M:%S %p')
		log_destination = os.environ.get('LOG_DESTINATION')
		log_handler = logging.StreamHandler()			

		if log_destination == PAPERTRAIL:
			log_handler = logging.handlers.SysLogHandler(address=('logs3.papertrailapp.com', 10024))
		elif log_destination == DATADOG:
			log_handler = DataDogLogHandler()
			
		log_handler.setFormatter(formatter)
		
		if not logger.handlers:
			logger.addHandler(log_handler)

		logger.propagate = False
		return logger

# Usage
logger = SingletonLogger.getInstance()



import os
import uuid

BID_STATUS = {
  'NEW': uuid.UUID('225d67d1-a733-45f6-8425-a6973bc1c0ac'),
  'ACCEPTED': uuid.UUID('9a0dc3e6-b451-4cf0-92e5-29d6e3695102'),
  'COUNTERED': uuid.UUID('d2261f74-2f48-4c01-a426-748ecb950912'),
  'DECLINED': uuid.UUID('269e4734-6ade-44a3-952b-6e619d0ac14b'),
  'INVITED': uuid.UUID('47102931-c5cd-4732-bb75-00e73c702e9e'),
  'SELECTED': uuid.UUID('5903a778-52bc-42cc-8e66-5dab8a8821c4'),
  'CANCELLED': uuid.UUID('88a8ba81-838a-43d3-86c5-d6d77240da71'),
  'EXPIRED': uuid.UUID('a030ad02-2903-48f5-83ec-d74208d44c35')
}

BOOKING_AGENT_URL = os.environ.get('BOOKING_AGENT_URL')
BOOKING_AGENT_TOKEN = os.environ.get("BOOKING_AGENT_TOKEN")
